<?php
namespace Ep\Cache\Adapter;

use Psr\Cache\CacheItemPoolInterface;
use Symfony\Component\Cache\Adapter;
use Zend\ServiceManager\AbstractPluginManager;
use Zend\ServiceManager\Factory\InvokableFactory;
use Zend\ServiceManager\ServiceManager;

/**
 * Plugin manager za CacheAdaptere.
 *
 * @package Ep\Cache\Adapter
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class PluginManager extends AbstractPluginManager
{
    /** @var string */
    protected $instanceOf = CacheItemPoolInterface::class;
    /** @var bool */
    protected $sharedByDefault = false;

    /** @var array */
    protected $aliases = [
        'apcu'       => Adapter\ApcuAdapter::class,
        'Apcu'       => Adapter\ApcuAdapter::class,
        'array'      => Adapter\ArrayAdapter::class,
        'Array'      => Adapter\ArrayAdapter::class,
        'chain'      => Adapter\ChainAdapter::class,
        'Chain'      => Adapter\ChainAdapter::class,
        'filesystem' => Adapter\FilesystemAdapter::class,
        'Filesystem' => Adapter\FilesystemAdapter::class,
        'FileSystem' => Adapter\FilesystemAdapter::class,
        'null'       => Adapter\NullAdapter::class,
        'Null'       => Adapter\NullAdapter::class,
        'phparray'   => Adapter\PhpArrayAdapter::class,
        'phpArray'   => Adapter\PhpArrayAdapter::class,
        'PhpArray'   => Adapter\PhpArrayAdapter::class,
        'phpfiles'   => Adapter\PhpFilesAdapter::class,
        'phpFiles'   => Adapter\PhpFilesAdapter::class,
        'PhpFiles'   => Adapter\PhpFilesAdapter::class,
        'proxy'      => Adapter\ProxyAdapter::class,
        'Proxy'      => Adapter\ProxyAdapter::class,
        'redis'      => Adapter\RedisAdapter::class,
        'Redis'      => Adapter\RedisAdapter::class,
    ];

    /** @var array */
    protected $factories = [
        Adapter\ApcuAdapter::class       => ApcuFactory::class,
        Adapter\ArrayAdapter::class      => ArrayFactory::class,
        Adapter\ChainAdapter::class      => ChainFactory::class,
        Adapter\FilesystemAdapter::class => FilesystemFactory::class,
        Adapter\NullAdapter::class       => InvokableFactory::class,
        Adapter\PhpArrayAdapter::class   => PhpArrayFactory::class,
        Adapter\PhpFilesAdapter::class   => FilesystemFactory::class,
        Adapter\ProxyAdapter::class      => ProxyFactory::class,
        Adapter\RedisAdapter::class      => RedisFactory::class,
    ];

    /**
     * PluginManager constructor.
     *
     * @param null $configInstanceOrParentLocator
     * @param array $config
     */
    public function __construct($configInstanceOrParentLocator = null, array $config = [])
    {
        parent::__construct($configInstanceOrParentLocator, $config);

        if ($configInstanceOrParentLocator instanceof ServiceManager) {
            $configInstanceOrParentLocator->setService(static::class, $this);
        }
    }
}