<?php
namespace Ep\InputFilter;

use Traversable;
use Zend\InputFilter\Exception;
use Zend\InputFilter\Factory as BaseFactory;
use Zend\InputFilter\InputFilterInterface;
use Zend\InputFilter\InputInterface;
use Zend\InputFilter\InputProviderInterface;
use Zend\Validator\ValidatorChain;

/**
 * Kreira input filtere.
 *
 * U specifikaciju Inputa je dodan ključ 'validator_chain'.
 *
 * @package Ep\InputFilter
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class Factory extends BaseFactory
{
	/**
     * Injecta naš InputFilter kao defaultnu klasu.
     *
	 * @param array|Traversable|\Zend\InputFilter\InputFilterProviderInterface $inputFilterSpecification
	 *
	 * @return InputFilterInterface
	 */
	public function createInputFilter($inputFilterSpecification)
	{
		if (is_array($inputFilterSpecification) && !isset($inputFilterSpecification['type'])) {
			$inputFilterSpecification['type'] = InputFilter::class;
		}

		return parent::createInputFilter($inputFilterSpecification);
	}

	/**
	 * U specifikaciji Inputa je sada moguće zadati klasu ValidatorChaina.
	 *
	 * @param  array|Traversable|InputProviderInterface $inputSpecification
     *
	 * @return InputInterface|InputFilterInterface
     *
     * @throws Exception\InvalidArgumentException
     * @throws Exception\RuntimeException
	 */
	public function createInput($inputSpecification)
	{
		$input = parent::createInput($inputSpecification);

		if (isset($inputSpecification['validator_chain'])) {
			$currentChain = $input->getValidatorChain();
			$newChain = $inputSpecification['validator_chain'];

			if (is_array($newChain)) {
				if (!isset($newChain['name'])) {
					throw new Exception\RuntimeException(
						'Invalid validator chain specification provided; does not include "name" key'
					);
				}

				$newChain = $currentChain->plugin($newChain['name'], $newChain['options'] ?? []);
			}

			if (is_string($newChain)) {
				$newChain = $currentChain->plugin($newChain);
			}

			if ($newChain instanceof ValidatorChain) {
				foreach ($currentChain->getValidators() as $validator) {
					$newChain->attach($validator['instance'], $validator['breakChainOnFailure']);
				}

				$input->setValidatorChain($newChain);
			}
		}

		return $input;
	}
}