<?php
namespace Ep\Log\Writer;

use Interop\Container\ContainerInterface;
use Zend\Log\Writer\Mail;
use Zend\Mail\Transport\TransportInterface;
use Zend\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu Mail LogWritera.
 *
 * @package Ep\Log\Writer
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class MailFactory implements FactoryInterface
{
    /**
     * Stvara LogWriter.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return Mail
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $options = (array) $options;

        if ($container->has('LogFilterManager')) {
            $options['filter_manager'] = $container->get('LogFilterManager');
        }

        if ($container->has('LogFormatterManager')) {
            $options['formatter_manager'] = $container->get('LogFormatterManager');
        }

        $transport = $options['transport'] ?? 'mail';

        if (!$transport instanceof TransportInterface) {
            if (!$container->has($transport)) {
                throw new \RuntimeException(sprintf(
                    "E-mail transport s imenom '%s' nije definiran.", $transport
                ));
            }

            $transport = $container->get($transport);
        }

        $options['transport'] = $transport;

        if (isset($_SERVER['HTTP_HOST']) && isset($options['subject_prepend_text'])) {
            $options['subject_prepend_text'] = sprintf($options['subject_prepend_text'], $_SERVER['HTTP_HOST']);
        }

        return new Mail($options);
    }
}