<?php
namespace Ep\View\Helper;

use Zend\Stdlib\ArrayUtils;
use Zend\View\Helper\AbstractHelper;
use Zend\View\Helper\BasePath;

/**
 * Vraća url resursa definiranog u AssetManifestu.
 *
 * @package Ep\View\Helper
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class AssetUrl extends AbstractHelper
{
    /** @var array */
    protected $options = [
        'manifest_path' => 'dist/manifest.json',
        'public_path' => 'dist',
    ];

    /** @var BasePath */
    protected $basePath;
    /** @var array */
    protected $manifest;

    /**
     * AssetUrl constructor.
     *
     * @param BasePath $basePath
     * @param array $options
     */
    public function __construct(BasePath $basePath, array $options = [])
    {
        $this->basePath = $basePath;
        $this->options = ArrayUtils::merge($this->options, $options);
    }

    /**
     * @param string $asset
     *
     * @return string
     */
    public function __invoke(string $asset): string
    {
        $manifest = $this->getManifest();
        $basePath = $this->basePath;

        if (!isset($manifest[$asset])) {
            throw new \RuntimeException(sprintf("Asset s imenom '%s' ne postoji u manifestu.", $asset));
        }

        $assetUrl = sprintf('%s/%s', rtrim($this->options['public_path'], '/'), ltrim($manifest[$asset], '/'));

        return $basePath($assetUrl);
    }

    /**
     * Vraća asset manifest.
     *
     * @return array
     */
    protected function getManifest(): array
    {
        if (null === $this->manifest) {
            if (!file_exists($this->options['manifest_path']) || !is_readable($this->options['manifest_path'])) {
                throw new \RuntimeException(sprintf(
                    "Datoteka '%s' ne postoji ili se ne može pročitati.", $this->options['manifest_path']
                ));
            }

            $json = file_get_contents($this->options['manifest_path']);
            $this->manifest = json_decode($json, true);

            if (null === $this->manifest) {
                throw new \RuntimeException(sprintf(
                    "Došlo je do greške prilikom dekodiranje manifesta: %s.", json_last_error_msg()
                ));
            }
        }

        return $this->manifest;
    }
}