<?php
namespace EpTest\Http\Validator;

use Ep\Http\Validator\JsonResponse as JsonResponseValidator;
use Ep\Http\Validator\JsonResponseFactory;
use EpTest\ContainerAwareTrait;
use PHPUnit\Framework\TestCase;
use Zend\Diactoros\Response;
use Zend\Diactoros\Response\HtmlResponse;
use Zend\Diactoros\Response\JsonResponse;

/**
 * Class JsonResponseValidatorTest
 *
 * @package EpTest\Http\Validator
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class JsonResponseTest extends TestCase
{
    use ContainerAwareTrait;

    /**
     * @expectedException \InvalidArgumentException
     */
    public function testWillOnlyValidateHttpResponses()
    {
        $validator = new JsonResponseValidator();
        $validator->isValid('');
    }

    public function testNonJsonResponsesAreInvalid()
    {
        $validator = new JsonResponseValidator();
        $response = new HtmlResponse('');

        $this->assertFalse($validator->isValid($response));
    }

    public function testMultiContentTypeIsDetected()
    {
        $validator = new JsonResponseValidator();
        $response = (new Response())
            ->withHeader('Content-Type', ['text/html', 'application/json']);

        $this->assertTrue($validator->isValid($response));
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testWillValidateRequiredFields()
    {
        $container = $this->createContainer();
        $factory = new JsonResponseFactory();

        $validator = $factory($container, '', [
            'schema' => [
                'status' => [
                    'required' => true,
                ],
            ],
        ]);

        $response = new JsonResponse([]);

        $this->assertFalse($validator->isValid($response));
        $this->assertArrayHasKey(JsonResponseValidator::INVALID_JSON, $validator->getMessages());
    }

    /**
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function testWillValidateWithValidators()
    {
        $container = $this->createContainer();
        $factory = new JsonResponseFactory();

        $validator = $factory($container, '', [
            'schema' => [
                'status' => [
                    'required' => true,
                    'validators' => [[
                        'name' => 'lessthan',
                        'options' => [
                            'max' => 400,
                        ],
                    ]],
                ],
            ],
        ]);

        $this->assertFalse($validator->isValid(new JsonResponse(['status' => 404])));
        $this->assertTrue($validator->isValid(new JsonResponse(['status' => 200])));
    }
}