<?php
namespace EpTest\Middleware;

use Ep\Middleware\ForceHttps;
use PHPUnit\Framework\TestCase;
use Zend\Diactoros\Response;
use Zend\Diactoros\ServerRequestFactory;
use Zend\Diactoros\Uri;
use Zend\Stratigility\MiddlewarePipe;

use function Zend\Stratigility\middleware;

/**
 * Class ForceHttpsTest
 *
 * @package EpTest\Middleware
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-framework
 */
class ForceHttpsTest extends TestCase
{
    public function testWillRedirectIfHttp()
    {
        $request = ServerRequestFactory::fromGlobals()
            ->withUri(new Uri('http://ep-middleware.localhost/path/script.php?param=value'));

        $pipeline = new MiddlewarePipe();
        $pipeline->pipe(new ForceHttps());

        $response = $pipeline->handle($request);

        $this->assertEquals(
            'https://ep-middleware.localhost/path/script.php?param=value',
            $response->getHeaderLine('Location')
        );
    }

    public function testWillPassThroughWhenHttps()
    {
        $request = ServerRequestFactory::fromGlobals()
           ->withUri(new Uri('https://ep-middleware.localhost'));

        $pipeline = new MiddlewarePipe();
        $pipeline->pipe(new ForceHttps());

        $pipeline->pipe(middleware(function() {
            return new Response\TextResponse('OK');
        }));

        $response = $pipeline->handle($request);

        $this->assertEquals('OK', $response->getBody()->getContents());
    }
}